#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>

#include <rlcsa/misc/utils.h>

#include "pdlrp.h"


int
main(int argc, char** argv)
{
  std::cout << "Building PDL-RP" << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: build_rp [-s] base_name" << std::endl;
    std::cout << "  -s  Use RePair for sets" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  bool use_rpset = false;
  if(argc > 2 && argv[1][0] == '-' && argv[1][1] == 's')
  {
    use_rpset = true;
    base_name = argv[2];
  }
  if(use_rpset) { std::cout << "Using RePair for sets" << std::endl; }
  std::cout << "Base name: " << base_name << std::endl;
  std::cout << std::endl;

  RLCSA rlcsa(base_name);
  if(!(rlcsa.isOk())) { return 2; }
  rlcsa.printInfo();
  usint rlcsa_bytes = rlcsa.reportSize(true);
  usint size = rlcsa.getSize();

  PDLRP rp(rlcsa, base_name, true, use_rpset);
  if(!(rp.isOk())) { return 3; }
  usint bytes = rp.reportSize();
  rp.writeTo(base_name, use_rpset);

  printSize("PDL", size, rlcsa_bytes, bytes);
  std::cout << std::endl;

  return 0;
}
