#ifndef _PDLTOPK_DOCLIST_H
#define _PDLTOPK_DOCLIST_H

#include <vector>

#include <rlcsa/rlcsa.h>

#include "pdltree.h"
#include "utils.h"


using CSA::RLCSA;
using CSA::usint;
using CSA::pair_type;


class PDLTopk
{
  public:
    typedef std::vector<Document>                   result_type;
    typedef enum { st_unfinished, st_error, st_ok } status_type;

    const static std::string EXTENSION; // .topk

    // If compress_sets is true, sets and grammar will be read from the files compressed by irepair.
    PDLTopk(const RLCSA& _rlcsa, const std::string& base_name, bool compress_sets);
    ~PDLTopk();

    inline bool isOk() const { return (this->status == st_ok); }

    void writeTo(const std::string& base_name) const;
    usint reportSize(bool include_freqs) const;

    // User must delete the result.
    result_type* query(const std::string& pattern, usint k, bool get_freqs) const;
    result_type* query(pair_type sa_range, usint k, bool get_freqs) const;

  private:
    const RLCSA&     rlcsa;
    status_type      status;

    PDLTree*         tree;
    CSA::MultiArray* freqs;
    CSA::ReadBuffer* grammar;
    CSA::MultiArray* blocks;

    result_type* queryUnsafe(pair_type sa_range, usint k, bool get_freqs) const;

    // These are not allowed.
    PDLTopk();
    PDLTopk(const PDLTopk&);
    PDLTopk& operator = (const PDLTopk&);
};


#endif  // _PDLTOPK_DOCLIST_H
