#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>

#include <rlcsa/misc/utils.h>

#include "pdltree.h"
#include "repair.h"


int
main(int argc, char** argv)
{
  std::cout << "RePair-based set compressor" << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: rpset base_name" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  std::cout << "Base name: " << base_name << std::endl;
  std::cout << std::endl;

  std::string sets_name = base_name + PDLTree::SETS_EXTENSION;
  std::ifstream input(sets_name.c_str(), std::ios_base::binary);
  if(!input)
  {
    std::cerr << "rpset: Cannot open input file " << sets_name << "!" << std::endl;
    return 1;
  }

  RePairEncoder::set_of_sets sets;
  RePairEncoder::set_type current;
  uint documents = 0; input.read((char*)&documents, sizeof(documents));
  while(!(input.eof()))
  {
    uint temp = 0; input.read((char*)&temp, sizeof(temp));
    if(temp > documents) { sets.push_back(current); current.clear(); }
    else { current.push_back(temp); }
  }
  input.close();

  RePairEncoder encoder(sets, true);
  if(!(encoder.isOk())) { return 2; }
  encoder.writeTo(base_name);

  return 0;
}
